﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using LogicData;

namespace SilverlightSocketClient.UIEvent
{
    public partial class AccountUI : UserControl
    {
        public AccountUI()
        {
            InitializeComponent();

            ChatTextBox.KeyDown += ChatBox_KeyDown;
            ChatButton.Click += btnChat_Click;
            LoginButton.Click += btnLogin_Click;
            RegisterButton.Click += btnRegister_Click;
            ActiveButton.Click += btnActiveCode_Click;
        }
        /// <summary>
        /// 로그인 요청을 서버로 전송한다.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnLogin_Click(object sender, RoutedEventArgs e)
        {
            Page.Instance.logicClient.SendLoginMsg(IDTextBox.Text, PWTextBox.Password);
        }
        /// <summary>
        /// 채팅을 하기 위해 버튼을 눌렀을 때 서버로 전송한다.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnChat_Click(object sender, RoutedEventArgs e)
        {
            Page.Instance.logicClient.SendChatMsg(ChatTextBox.Text, ""/*NickName은 서버에서 로그인 된 아이디로 보내준다*/);
            ChatTextBox.Text = "";
        }
        /// <summary>
        /// 채팅창에서 엔터를 눌렀을 때 버튼과 같은 효과를 내도록 한다.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ChatBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter)
            {
                btnChat_Click(sender, null);
            }
        }        /// <summary>
        /// 회원 가입 버튼을 눌렀을 때 이벤트
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRegister_Click(object sender, RoutedEventArgs e)
        {
            Page.Instance.logicClient.SendRegisterMsg(RegIDTextBox.Text, EMailTextBox.Text);
        }
        /// <summary>
        /// 회원 가입시 메일로 받은 ActiveCode를 입력하고 승인받기 위한 메세지를 보낸다.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnActiveCode_Click(object sender, RoutedEventArgs e)
        {
            Page.Instance.logicClient.SendActiveCodeMsg(ActiveIDTextBox.Text, ActiveCodeTextBox.Text);
        }
        public void ChatListBoxAdd(string chatMsg, string nickName, ChatType type)
        {
            ListBoxItem listbox = new ListBoxItem();
            listbox.Content = "[" + nickName + "] " + chatMsg;
            if (type == ChatType.SystemChat)
            {
                if (nickName == "SYSTEM")
                {
                    SolidColorBrush blueColor = new SolidColorBrush();
                    blueColor.Color = Color.FromArgb(255, 0, 0, 255);
                    listbox.Foreground = blueColor;
                    listbox.FontWeight = FontWeights.Bold;
                }
                else if (nickName == "ERROR")
                {
                    SolidColorBrush redColor = new SolidColorBrush();
                    redColor.Color = Color.FromArgb(255, 255, 0, 0);
                    listbox.Foreground = redColor;
                    listbox.FontWeight = FontWeights.ExtraBold;
                }
                else
                {
                    SolidColorBrush blackColor = new SolidColorBrush();
                    blackColor.Color = Color.FromArgb(255, 255, 255, 255);
                    listbox.Foreground = blackColor;
                    listbox.FontWeight = FontWeights.Normal;
                }
            }
            ChatListBox.Items.Add(listbox);
        }
        public void LoginSuccess()
        {
            ChatTextBox.Text = "";
            //LoginSuccess.Begin();
            //LoginSuccess.Completed += new EventHandler(LoginSuccessAnimationCompleted);
        }

        void LoginSuccessAnimationCompleted(object sender, EventArgs e)
        {

        }
    }
}
