﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Text;
using System.Net;
using System.Threading;

using Network;

namespace GameLogic
{
    sealed class LogicManager
    {
        /// <summary>
        /// 싱글톤 선언 코드
        /// </summary>
        private static volatile LogicManager instance;
        private static object syncRoot = new Object();

        private LogicManager() 
        {
            ClientList = new Dictionary<string, Client>();
            Util = new LogicUtil();
            emailSend = new EMailSend();
        }

        public static LogicManager Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                            instance = new LogicManager();
                    }
                }

                return instance;
            }
        }
        // 싱글톤 끝

        // 유저 정보. 접속 정보나 기타 자료들이 다 들어있다.
        public Dictionary<string, Client> ClientList;
        // 유용한 함수 등이 모여있는 클래스
        public LogicUtil Util;
        // 서버
        public NetworkServer server;
        // 메일 보내기 로직
        public EMailSend emailSend;

        public void ServerStart()
        {
            NetworkMsg msg = new NetworkMsg();
            server = new NetworkServer(msg);
            server.Start();
        }


    }
}